/* SPDX-License-Identifier: LGPL-3.0-or-later */

/*
 * Copyright (C) 2023 Perry Werneck <perry.werneck@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef FIELD_H_INCLUDED

 #define FIELD_H_INCLUDED
 #pragma once

 #include <lib3270/ipc.h>
 #include <lib3270/ipc/session.h>

 namespace TN3270 {

		/// @brief TN3270 Action
		class TN3270_PUBLIC Field {
		protected:
			TN3270::Session &session;	///< @brief The associated session
			unsigned short start;		///< @brief Address of field start.
			unsigned short length;		///< @brief Field length.

			constexpr Field(TN3270::Session &h, unsigned short s, unsigned short l) : session{h}, start{s}, length{l} {
			}

		public:

			/// @brief Jump to next unprotected field.
			/// @return true if the field is valid.
			virtual bool next() = 0;

			/// @brief Get Field contents.
			inline std::string get() const {
				return session.toString(start,length);
			}

			/// @brief Set Field Contents.
			void set(const std::string &str) {
				session.push(start,str.c_str(),length);
			}


		};

	}

#endif // FIELD_H_INCLUDED
